<?php

namespace Drupal\access_by_entity\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\user\RoleStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\access_by_entity\AccessByEntityStorageInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * AccessByEntityForm.
 */
class AccessByEntityForm extends FormBase {

  /**
   * The role storage.
   *
   * @var \Drupal\user\RoleStorageInterface
   */
  protected $roleStorage;

  /**
   * Access By Entity Storage.
   *
   * @var \\Drupal\access_by_entity\AccessByEntityStorageInterface
   */
  protected $accessByEntityStorage;

  /**
   * Request stack.
   *
   * @var Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new AccessByEntityForm.
   *
   * @param \Drupal\user\RoleStorageInterface $role_storage
   *   The role storage.
   * @param \Drupal\access_by_entity\AccessByEntityStorageInterface $access_by_entity_storage
   *   The module handler.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The module handler.
   * @param \Drupal\Core\Routing\RouteMatchInterface $current_route_match
   *   The core routeMatch service.
   */
  public function __construct(RoleStorageInterface $role_storage,
    AccessByEntityStorageInterface $access_by_entity_storage,
    RequestStack $request_stack,
    RouteMatchInterface $current_route_match
  ) {
    $this->roleStorage = $role_storage;
    $this->accessByEntityStorage = $access_by_entity_storage;
    $this->requestStack = $request_stack;
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('user_role'),
      $container->get('access_by_entity.access_storage'),
      $container->get('request_stack'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_admin_access_by_entity';
  }

  /**
   * Gets the roles to display in this form.
   *
   * @return \Drupal\user\RoleInterface[]
   *   An array of role objects.
   */
  protected function getRoles() {
    return $this->roleStorage->loadMultiple();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $parameters = $this->currentRouteMatch->getRouteObject()->getOption('parameters');
    $entity_type_id = array_keys($parameters)[0];
    $entity = $this->requestStack->getCurrentRequest()->attributes->get(
      $entity_type_id
    );
    $role_names = [];
    foreach ($this->getRoles() as $role_name => $role) {
      // Retrieve role names for columns.
      if (!$role->isAdmin()) {
        $role_names[$role_name] = $role->label();
      }
    }
    // Store $role_names for use when saving the data.
    $form['role_names'] = [
      '#type' => 'value',
      '#value' => $role_names,
    ];
    $form['permissions'] = [
      '#type' => 'table',
      '#header' => [$this->t('Permission')],
      '#id' => 'permissions',
      '#attributes' => ['class' => ['permissions', 'js-permissions']],
      '#sticky' => TRUE,
    ];
    foreach ($role_names as $name) {
      $form['permissions']['#header'][] = [
        'data' => $name,
        'class' => ['checkbox'],
      ];
    }
    $permissions = [
      'view' => [
        'title' => $this->t('View content'),
        'description' => NULL,
        'provider' => 'entity',
      ],
      'edit' => [
        'title' => $this->t('Edit content'),
        'description' => NULL,
        'provider' => 'entity',
      ],
      'delete' => [
        'title' => $this->t('Delete content'),
        'description' => NULL,
        'provider' => 'entity',
      ],
    ];
    $default_values = $this->accessByEntityStorage->findBy(
      [
        ['key' => 'entity_id', 'value' => $entity->id()],
        ['key' => 'entity_type_id', 'value' => $entity_type_id],
      ]
    );
    $default_values_permission = [];
    foreach ($default_values as $item) {
      if (!isset($default_values_permission[$item->perm])) {
        $default_values_permission[$item->perm] = [$item->rid => $item->rid];
      }
      else {
        $default_values_permission[$item->perm][$item->rid] = $item->rid;
      }
    }

    foreach ($permissions as $perm => $perm_item) {
      // Fill in default values for the permission.
      $perm_item += ['restrict access' => FALSE];
      $form['permissions'][$perm]['description'] = [
        '#type' => 'inline_template',
        '#template' => '<div class="permission"><span class="title">{{ title }}</span></div>',
        '#context' => [
          'title' => $perm_item['title'],
        ],
      ];

      foreach ($role_names as $rid => $name) {
        $form['permissions'][$perm][$rid] = [
          '#title' => $name . ': ' . $perm_item['title'],
          '#title_display' => 'invisible',
          '#wrapper_attributes' => [
            'class' => ['checkbox'],
          ],
          '#type' => 'checkbox',
          '#default_value' => isset($default_values_permission[$perm][$rid]),
          '#attributes' => [
            'class' => [
              'rid-' . $rid,
              'js-rid-' . $rid,
            ],
          ],
          '#parents' => [$rid, $perm],
        ];
      }
    }

    $form['entity_id'] = [
      '#type' => 'hidden',
      '#default_value' => $entity->id(),
    ];

    $form['entity_type_id'] = [
      '#type' => 'hidden',
      '#default_value' => $entity_type_id,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save permissions'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $entity_id = $form_state->getValue('entity_id');
    $entity_type_id = $form_state->getValue('entity_type_id');
    $this->accessByEntityStorage->clear($entity_id, $entity_type_id);
    foreach ($form_state->getValue('role_names') as $role_name => $name) {
      $this->accessByEntityStorage->save(
        $entity_id, $entity_type_id, $role_name,
        array_diff((array) $form_state->getValue($role_name), [0])
      );
    }
    drupal_set_message($this->t('The changes have been saved.'));
  }

}
